/**
 * =====================================================
 * Relation.java: .....
 *
 * @author Willy Tiengo
 * =====================================================
 */

package osm02.model;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.geom.LineString;
import com.vividsolutions.jts.geom.MultiLineString;
import com.vividsolutions.jts.geom.Polygon;
import com.vividsolutions.jts.io.WKBWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Map;

public class Relation extends AbstractNode {
    public  List<Member>   members;

    public Relation(String id, String visible, String timestamp,
        String version, String changeset, String user,
        String uid, List<Member> members, Map<String, String> tags) {

        super(id, visible, timestamp, version, changeset, user, uid, tags);

        if ( tags != null )
           this.tags = tags;

        this.members = members;
    }

    // Add OSM member to Relation model ....

    public void addMember ( Member member ) {

       // Create list for storage of nodes ....

       if ( members == null ) {
          members = new ArrayList<Member>();
       }

       // Add member item to array list ...

       members.add ( member );
    }

    /**
     * @return The MultiLineString of all ways members of this relation. If any
     *         way members can not be found in the datase, returns
     *         <code>null</code>.
     */

    public void getPolygon() {
       // ... details removed ....
    }

    // Retrieve specific tags ....

    public boolean isRoute() {
        return tags.get("route") != null;
    }

    public boolean isBoundary() {
        return tags.get("boundary") != null;
    }

    public int getAdminLevel() {
        return Integer.parseInt(tags.get("admin_level"));
    }

    public List<Member> getMembers() {
       return members;
    }

    public String getName() {
        return tags.get("name");
    }

    public String getType() {
        return tags.get("type");
    }

    private boolean isWay(Member m) {
        return m.type.equals("way");
    }

    // =======================================================================
    // Create string representation of a Relation() .....
    // =======================================================================

    public String toString() {
       StringBuffer buf = new StringBuffer();
       buf.append( String.format("*** Relation[ name = %s ] ... \n", this.getName() ) );
       buf.append( String.format("               key = %s ... \n", getId() ) );
       buf.append( String.format("              type = %s ... \n", this.getType() ) );

       String s = (String) tags.get(ROUTEMASTER);
       if( s != null) 
         buf.append( String.format("      route_master = %s ... \n", s ));

       if( members != null ) {
          int i = 1;
          for (Member member : members) {
             buf.append( String.format("*** Member [%3d]: type = %s: ref = %s: role = %s ... \n",
                                            i, member.type, member.ref, member.role ) );
             i = i + 1;
          }
       }

       return buf.toString();
    }
}
