/*
 *  ===========================================================================
 *  DOMPlanner.java: A day planner application using DOM.
 *  
 *  Code developed by Deitel and Deitel, 2000.
 *  Code slightly modified for JAXP: By Mark Austin                 March, 2002
 *  Function AddSchedule: Added by Vimal and Deep                     May, 2002
 *  ===========================================================================
 */

package planner02;

import java.io.*;
import java.awt.*;
import java.util.*;
import javax.swing.*;

import org.w3c.dom.*;
import org.xml.sax.*;
import javax.xml.parsers.*;
import java.text.*;

import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;

public class DOMPlanner {
  
   private JTextArea display;    // for displaying output
   private InputSource input;    // for reading the XML document
   private Document document;    // document node object

   // Variables to store the query parameters and the result

   private int year, month, day, timePeriod; 
   private String resultYear, resultDay;

   public DOMPlanner( JTextArea output ) {

      year    = month = day = timePeriod = -1;
      display = output; 

      try {

         // Obtain the default parser

         DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
         factory.setValidating( true );
         DocumentBuilder builder = factory.newDocumentBuilder();

         // Set error handler for validation errors

         builder.setErrorHandler( new MyErrorHandler() );

         // Obtain Document Object from XML Document

         document = builder.parse( new File( "data/planner.xml" ) );
      } 

      // Catch errors .....

      catch ( SAXParseException spe ) {
         System.err.println( "Parse error: " + 
            spe.getMessage() );
         System.exit( 1 );
      }

      catch ( SAXException se ) {
         se.printStackTrace();         
      }

      catch ( FileNotFoundException fne ) {
         System.err.println( "File \"data/planner.xml\" not found." );
         System.exit( 1 );
      }

      catch ( Exception e ) {
         e.printStackTrace();
      }
   }


   public void addSchedule( int y, int m, int d, int t, String description ) {
	   Element rootElem = document.getDocumentElement();
	   NodeList yearNodes = rootElem.getElementsByTagName("year");
	   
	   // find if the result matches the year input y
	   for(int ii=0; ii<yearNodes.getLength(); ii++)
	   {
		   NamedNodeMap yearAttributes = yearNodes.item(ii).getAttributes();
		   Node value = yearAttributes.item(0);
		   if(Integer.parseInt(value.getNodeValue()) == y)
		   {
			   // We got a match in the XML file for the year
			   // Create a date node with month and day attribute and set value
			   Element dateElem = document.createElement("date");
			   Attr monthAttr = document.createAttribute("month");
			   monthAttr.setValue((String) "" +m);
			   dateElem.setAttributeNode(monthAttr);
			   Attr dayAttr = document.createAttribute("day");
			   dayAttr.setValue((String)"" +d);
			   dateElem.setAttributeNode(dayAttr);
			   
			   // Create a "note" node with "time" attribute and set value
			   Element noteElem = document.createElement("note");
			   Attr timeAttr = document.createAttribute("time");
			   timeAttr.setValue((String)""+t);
			   noteElem.setAttributeNode(timeAttr);
			   
			   noteElem.appendChild(document.createTextNode(description));
			   
			   // finally add the "date" node inside the "year" node
			   // and "note" node inside the "date" node
			   yearNodes.item(ii).appendChild(dateElem);
			   dateElem.appendChild(noteElem);
		   }
	   }
	   
	   // Here modify the xml file. Please note that class XmlDocument
	   // has been migrated to package org.apache.crimson.tree instead
	   // of com.sun.java.tree package as written in Deitel book and 
	   // lots of other places in April 2000

	   try {
                TransformerFactory transformerFactory = TransformerFactory.newInstance();
                Transformer serializer = transformerFactory.newTransformer();
                serializer.transform( new DOMSource(document),
                                      new StreamResult(
                                      new FileOutputStream("planner.xml" )) );
           }
	   catch (Exception e) {
		   e.printStackTrace();
	   }
   }  // End Function addSchedule
   
   public void getQueryResult(int y, int m, int d, String input) {
	   year = y;
	   month = m;
	   day = d;
	   String monthNames[]={" ", "JAN","FEB","MAR","APR","MAY","JUNE","JULY",
						    "AUG","SEP","OCT","NOV","DEC"};
       boolean flag = false;

		if(input == "MONTH")
		{
			// Display schedule for entire month. Set contents of textArea accourdingly
			display.setText("****** Schedule for the month of " + monthNames[month] + ", " + year + " ******" );
			display.append("\n\n");
			
			Element rootElem = document.getDocumentElement();
			// Retrieve all the "year" nodes as a list
			NodeList yearNodeList = rootElem.getElementsByTagName("year");
			
			for(int ii=0; ii<yearNodeList.getLength(); ii++)
			{
				   Element currentYearNode = (Element) yearNodeList.item(ii);

				   // Retrieve the "value" attribute for the "year" node
				   NamedNodeMap yearAttributes = currentYearNode.getAttributes();
				   Node value = yearAttributes.item(0);
				   // Check if there is a match b/w input year and current year node
				   if(Integer.parseInt(value.getNodeValue()) == year)
				   {
					   // Year match found. Retrieve the "date" nodes inside this "year" node
					   NodeList dateNodeList = currentYearNode.getElementsByTagName("date");
					   
					   // This marker is used so that we display the title only once..
					   // notice its use below.
					   boolean marker = false;
					   for(int jj=0; jj<dateNodeList.getLength(); jj++)
					   {
						    Element currentDateNode = (Element) dateNodeList.item(jj);
							if(Integer.parseInt(currentDateNode.getAttribute("month")) == month) {
								// "month" attribute matches the input month. Display the title of the text area
								if(marker == false) {
									display.append(" Date   Time             Activity\n");
									display.append("======================================================\n");
									marker = true;
								}
									
									
								// Retrieve the day value and the "note" nodes
								day = Integer.parseInt( currentDateNode.getAttribute( "day" ) );
								NodeList noteNodeList = currentDateNode.getElementsByTagName("note");
								
								for(int kk=0; kk<noteNodeList.getLength(); kk++) {
									// flag is used to set the title "no schedule for the given month"
									// if its value remains false for the entire function.
									flag = true;
									Element currentnoteNode = (Element) noteNodeList.item(kk);
									timePeriod = Integer.parseInt(currentnoteNode.getAttribute("time"));
									
									NodeList childNode = currentnoteNode.getChildNodes();
									Text text = (Text) childNode.item(0);
									// Format the output and display it on text area
									DecimalFormat dayFormatter = new DecimalFormat("00");
									DecimalFormat timeFormatter = new DecimalFormat("0000");
									display.append(dayFormatter.format(day) +" " + monthNames[month] 
												   +"  " + timeFormatter.format(timePeriod) + "  "
												   + text.getNodeValue().trim() +"\n");
								}
							} 
					   } 
				   }
			}
			if(!flag) 
				display.append("\tNo activity scheduled for this month.. \n\tUse Add Schedule button to add activity");
		}
		if(input == "DAY")
		{
			// This block is written on the similar lines except for the fact that
			// now we need to check for both "day" and "month" attribute to see
			// if it matches the input query.
			display.setText("****** Schedule for " + day + " "  + monthNames[month] + ", " + year + " ******" );
			display.append("\n\n");
			
			Element rootElem = document.getDocumentElement();
			NodeList yearNodeList = rootElem.getElementsByTagName("year");
			
			for(int ii=0; ii<yearNodeList.getLength(); ii++)
			{
				   Element currentYearNode = (Element) yearNodeList.item(ii);

				   NamedNodeMap yearAttributes = currentYearNode.getAttributes();
				   Node value = yearAttributes.item(0);
				   if(Integer.parseInt(value.getNodeValue()) == year)
				   {
					   boolean marker = false;
					   
					   NodeList dateNodeList = currentYearNode.getElementsByTagName("date");
					   
					   for(int jj=0; jj<dateNodeList.getLength(); jj++)
					   {
						    Element currentDateNode = (Element) dateNodeList.item(jj);
							if(Integer.parseInt(currentDateNode.getAttribute("month")) == month) {
								
								if(Integer.parseInt(currentDateNode.getAttribute("day")) == day )
								{
									if (marker == false) {
											display.append("Time          Activity\n");
											display.append("====================================\n");
											marker = true;
									}
										
										
									NodeList noteNodeList = currentDateNode.getElementsByTagName("note");
								
									for(int kk=0; kk<noteNodeList.getLength(); kk++) {
										flag = true;
										Element currentnoteNode = (Element) noteNodeList.item(kk);
										timePeriod = Integer.parseInt(currentnoteNode.getAttribute("time"));
										
										NodeList childNode = currentnoteNode.getChildNodes();
										Text text = (Text) childNode.item(0);
										DecimalFormat timeFormatter = new DecimalFormat("0000");
										display.append(timeFormatter.format(timePeriod) + "  " + text.getNodeValue().trim() +"\n");
									}
								}
							}
					   }
				   }
			}
			if(!flag) 
				display.append("\tNo activity scheduled for this date.. \n\tUse Add Schedule button to add activity");

		}
	} // End function getQueryResult
   
} // End Class DOMPLanner


								

						   
					   
			
			
  
   
   
   
   
   
   
   
   
