/*
 *  ===========================================================================
 *  DayPlanner.java: Program for GUI interface for the day planner application.
 *  
 *  Code developed by Deitel and Deitel, 2000.
 *  Code slightly modified for JAXP: By Mark Austin                 March, 2002
 *  Code modified to implement Add Schedule module and Swing Calendar interface
 *  
 *  By: Vimal and Deep                                              May 2002
 *  ===========================================================================
 */

package planner02;

import java.awt.*;                  
import java.awt.event.*;            
import javax.swing.*;               
import javax.swing.event.*;
import java.util.Calendar;

public class DayPlanner extends JFrame implements ActionListener, MouseListener {

   // GUI components

   private JComboBox yearCombo, monthBox, yearBox; // Add timeCombo later here if need arises
	private JButton monthButton;
	private JButton tempButton[];
	private JPanel panel1, panel2;
	private DOMPlanner handler;
	private String monthNames[]={"JANUARY","FEBRUARY","MARCH","APRIL","MAY","JUNE","JULY",
                                      "AUGUST","SEPTEMBER","OCTOBER","NOVEMBER","DECEMBER"};
	private int selMonth, selYear, selDate, comboYearIndex;
	private String yearValues[] = {"2017","2018","2019"};
	private JDialog addSchedDialog;
	private JTextField addSchText;
	
	// This class is used to input only integers in the date field in add Schedlue dialog box
	WholeNumberField dateValueField, timeValueField;
  
    public DayPlanner() {
	    super( "DOM-Based Travel Planner" );
		Font font = new Font( "Monospaced", java.awt.Font.BOLD, 16 );

		JTextArea display   = new JTextArea(10,1);
		display.setFont( font );
		display.setEditable( false );
		display.setBackground(new Color(230, 230, 250)); // Lavendar color
     
		handler = new DOMPlanner( display );

		// Swing Calendar Interface construction begins here
		JButton forwardButton = new JButton(new ImageIcon("data/FastForward16.gif"));
		forwardButton.setActionCommand(">>");
		forwardButton.addActionListener(this);
		
		JButton backButton = new JButton(new ImageIcon("data/Rewind16.gif"));
		backButton.setActionCommand("<<");
		backButton.addActionListener(this);
		
		// Get an instance of the current system date using the Calendar class
		Calendar curCal = Calendar.getInstance();
		selMonth = curCal.get(curCal.MONTH);
		selYear = curCal.get(curCal.YEAR);
		
		monthButton = new JButton(monthNames[selMonth]);
		monthButton.setActionCommand("MONTH");
		monthButton.setToolTipText("Click here to see the schedule for this month");
		monthButton.addActionListener(this);
		
		// Initialized the ContentPane and set the layout to BorderLayout
		Container container = getContentPane();
		container.setLayout(new BorderLayout());
		
		JPanel northPanel = new JPanel(true); // Overlay panel for the JFrame CENTER
		northPanel.setLayout(new BorderLayout());		
		northPanel.add(new JLabel("        "), BorderLayout.EAST);
		northPanel.add(new JLabel("        "), BorderLayout.WEST);
		northPanel.add(new JScrollPane(display), BorderLayout.CENTER);
		container.add(northPanel, BorderLayout.NORTH);
		
		// Overlay panel for the JFrame CENTER
		panel1 = new JPanel(true); 
		panel1.setLayout(new BorderLayout());
		// for back, Month and forward and year buttons
		JPanel panel3 = new JPanel(true); 
		// Date Panel For drawing the date mesh
		panel2 = new JPanel(true); 
		
		panel3.setLayout(new FlowLayout());
		panel3.add(backButton);
		panel3.add(monthButton);
		panel3.add(forwardButton);

		yearCombo = new JComboBox( yearValues );
		
		// Set the current state of the combo box based on the current year
		for(int ii=0; ii<yearValues.length; ii++)
			if(Integer.parseInt(yearValues[ii]) == selYear)
				yearCombo.setSelectedIndex(ii);  
		
		// We update the date panel if user changes the ComboBox value
		yearCombo.addActionListener(this); 
		panel3.add(yearCombo);
		panel1.add(panel3, BorderLayout.NORTH);
		
		tempButton = new JButton[31];

		panel2 = GetDatePanel(selMonth, selYear);
		panel1.add(panel2, BorderLayout.CENTER);
		
		// These empty labels are specified for date mesh overall padding
		panel1.add(new JLabel("            "), BorderLayout.EAST);
		panel1.add(new JLabel("            "), BorderLayout.WEST);
		
		container.add(panel1, BorderLayout.CENTER);
		
		JButton addSchedButton = new JButton("Add Schedule");
		addSchedButton.addActionListener(this);
		
		JPanel southPanel = new JPanel(true); // Overlay panel for the JFrame CENTER
		southPanel.setLayout(new BorderLayout());		
		southPanel.add(new JLabel("        "), BorderLayout.EAST);
		southPanel.add(new JLabel("        "), BorderLayout.WEST);
		southPanel.add(addSchedButton, BorderLayout.CENTER);
		container.add(southPanel, BorderLayout.SOUTH);
		
		// Set the size of the frame and display it
		setSize(650,500);
		show();
		
	} // End Constructor DayPlanner()
	
	boolean IsLeapYear(int year) 
	{
			/* If multiple of 100, leap year iff multiple of 400. */    
			if ((year % 100) == 0) return((year % 400) == 0);      
			/* Otherwise leap year iff multiple of 4. */    
			return ((year % 4) == 0);    
	} // IsLeapYear  

	int NumberRowsNeeded(int firstDayOfMonth, int year, int month)    {
		int noDays[] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

		int numCells; 
		/* number of cells needed by the month */
		/* Non leap year February with 1st on Sunday: 4 rows. */
		if ((month == 1) && (firstDayOfMonth == 1) && !IsLeapYear(year)) return (5);
		/* Number of cells needed = blanks on 1st row + days in month. */	
		numCells = firstDayOfMonth + noDays[month] -1 ;
		/* One more cell needed for the Feb 29th in leap year. */
		if ((month == 1) && (IsLeapYear(year))) numCells++;
		/* 35 cells or less is 5 rows; more is 6. */
		return ((numCells <= 35) ? 6 : 7);    
	} // NumberRowsNeeded  

	JPanel GetDatePanel(int month, int year)
	{
		int noDays[] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
    	String dayNames[] = {"SUN", "MON", "TUE", "WED", "THU", "FRI", "SAT"};
		
		Calendar cloneCalendar = Calendar.getInstance();
		cloneCalendar.set(year, month, 1);
		if(IsLeapYear(year)) 	noDays[1]=29;
		
		int firstDayOfMonth = cloneCalendar.get(cloneCalendar.DAY_OF_WEEK);
		JPanel datePanel = new JPanel();
		int noOfRows = NumberRowsNeeded(firstDayOfMonth, year, month);
	    datePanel.setLayout(new GridLayout(noOfRows ,7));
		int counter = 1;
		for (int ii=1; ii<=7*noOfRows; ii++) {
			if(ii<=7) 
			{
				JButton dateButton = new JButton(dayNames[ii-1]);
				//dateButton.setBackground(new Color(255, 250, 205));
				dateButton.setBackground(Color.yellow);
				//dateButton.setBorder(BorderFactory.createEtchedBorder(Color.black, Color.black));
				dateButton.setBorder(BorderFactory.createMatteBorder(2,1,2,1,Color.black));
				
				datePanel .add(dateButton);	
			}
			else {		
				
		    	if((ii>=7+firstDayOfMonth) && (counter <= noDays[month])) 
				{
					tempButton[ii-7-firstDayOfMonth] = new JButton("" + (ii-6-firstDayOfMonth));
					tempButton[ii-7-firstDayOfMonth].setForeground(Color.yellow);
					tempButton[ii-7-firstDayOfMonth].setBackground(new Color(106,90,205));
					tempButton[ii-7-firstDayOfMonth].setToolTipText("Click here to see the schedule for this date");
					
					tempButton[ii-7-firstDayOfMonth].addActionListener(this);
					tempButton[ii-7-firstDayOfMonth].addMouseListener(this);
					tempButton[ii-7-firstDayOfMonth].setBorder(BorderFactory.createRaisedBevelBorder());

					datePanel.add(tempButton[ii-7-firstDayOfMonth]);
					//System.out.println(" ii: " + ii + " tempButton " + (ii-7-firstDayOfMonth) + " added");
					counter++;
				}
				else
				{
					JButton emptyButton = new JButton("");
					emptyButton.setForeground(Color.yellow);
					emptyButton.setBackground(new Color(106,90,205));
					emptyButton.setBorder(BorderFactory.createRaisedBevelBorder());
					datePanel.add(emptyButton);
				}
			} // End if Loop	
		} // End for Loop
	    return datePanel;
	} // End function DrawDatePanel()
	
	public void actionPerformed( ActionEvent e ) {
		
		// Catch the "Forward" button clicking here
		if ( e.getActionCommand() == ">>" ) { 
			++selMonth;
			if(selMonth == 12) {
				if(comboYearIndex == 2) {selMonth--; return;} // End limit to three year plannar is reached.
				else {selMonth=0; ++selYear; ++comboYearIndex; yearCombo.setSelectedIndex(comboYearIndex);} // also change the combo box value write code here
			}
			monthButton.setText(monthNames[selMonth]);
	            panel1.remove(panel2);
				panel2 = GetDatePanel(selMonth, selYear);
				panel1.add(panel2, BorderLayout.CENTER);
		}
		
		// Catch the "Back" button clicking here
		if (e.getActionCommand() == "<<" ) {
	         --selMonth;
			 
			if(selMonth == -1) {
				if(comboYearIndex == 0) {selMonth++; return; } // NO data avaiable for the past year
	                  else { selMonth =11; --selYear; --comboYearIndex; yearCombo.setSelectedIndex(comboYearIndex); } // also change the combo box value write code here 
			}
			monthButton.setText(monthNames[selMonth]);
			panel1.remove(panel2);
			panel2 = GetDatePanel(selMonth, selYear);
			panel1.add(panel2, BorderLayout.CENTER);
		}
		
		// Display schedule for the month if the user has clicked on month
		if(e.getActionCommand() == "MONTH") {
			handler.getQueryResult( selYear, selMonth+1, -1, "MONTH" );
		}
		
		// Change date panel to reflect the change in the year selection
	    if(e.getSource() == yearCombo) {
			String selYearString = (String) yearCombo.getSelectedItem();
			selYear = Integer.parseInt(selYearString);
			// This is the tricky part. Missing any of the step will result
			// in the Date mesh not being updated.
			panel1.remove(panel2);
			panel2 = new JPanel();
			panel2 = GetDatePanel(selMonth, selYear);
			panel1.add(panel2, BorderLayout.CENTER);
			panel1.updateUI();
		}
		
		if (e.getActionCommand() == "Add Schedule") {
			// Here we start constructing the Add Schedule dialog
			// the last argument is specified true is the modality feature of the JDialog
			// class which prohibits user to enter any inputs to the parent window
			addSchedDialog = new JDialog((Frame)null,"Add Schedule to the planner", true);
			
			ConstructAddSchedDialog(addSchedDialog);
			addSchedDialog.setSize(500,150);
			addSchedDialog.setVisible(true);
		}
		
		// Display schedule for a particular day
		for(int ii=0; ii<tempButton.length; ii++)
		{
			if(e.getSource() == tempButton[ii])
				handler.getQueryResult( selYear, selMonth+1, ii+1, "DAY" );
		}
		
		if(e.getActionCommand() == "CANCEL") {
			// Do nothing.. just hide the addSchedule Dialog box
			addSchedDialog.setVisible(false);
		}
		
		if(e.getActionCommand() == "OK") {
			int selMonthIndex = monthBox.getSelectedIndex();
			String selYearIndexString = (String) yearBox.getSelectedItem();
			int selYearIndexInt = Integer.parseInt(selYearIndexString);
			int selDate = dateValueField.getValue();
			int selTime = timeValueField.getValue();
			
			// Check for the invalid date if entered by the user
			if(!isDateValid(selYearIndexInt, selMonthIndex, selDate))
		    {
				// Pop up a dialog showing user has entered invalid date
				InfoDialog myDialog = new InfoDialog(addSchedDialog, "Invalid Date Entered");
				return;
			}
			
			// check for invalid time
			if(!isTimeValid(selTime))
			{
				// Pop up a dialog here showing user has entered invalid time
				InfoDialog myDialog = new InfoDialog(addSchedDialog, "Invalid Time Entered");
				return;
			}
			
			handler.addSchedule( selYearIndexInt, selMonthIndex+1, selDate, selTime, (String) addSchText.getText() );
			InfoDialog myDialog = new InfoDialog(addSchedDialog, "Schedule added successfully ");
			addSchedDialog.setVisible(false);
		}
	} // End fuction actionPerformed	
	
	public static void main( String s[] ) {
		// DayPlanner program execution begins here. 
		
		// Here we set the look and feel of the application as the platform
		// dependent. In the absence of this code Java look and feel is 
		// implemented as default.		

		UIManager myManager = new UIManager();
		try {
			myManager.setLookAndFeel(myManager.getSystemLookAndFeelClassName());
		}
		catch (Exception e) {e.printStackTrace(); }
			
		// Create instance of DayPlanner().....
		DayPlanner d = new DayPlanner();

		// Setup (inner class) listener to close window.....

		d.addWindowListener(
		   new WindowAdapter() {
		      public void windowClosing( WindowEvent e ) {
		         System.exit( 0 );
		      }
		   }
		);
	} // End Function main()
	
	void ConstructAddSchedDialog(JDialog addSchedDialog)
	{
		// This function constructs the Add Schedule dialog box which pops up
		// as the user clicks on the Add Schedule button in the main window.
		monthBox = new JComboBox(monthNames);
		yearBox = new JComboBox(yearValues);
		
		Calendar curCal = Calendar.getInstance();
		int monthValue = curCal.get(curCal.MONTH);
		monthBox.setSelectedIndex(monthValue);
		int curYear = curCal.get(curCal.YEAR);
		String curYearString = ("" +curYear);
		int curDate = curCal.get(curCal.DATE);
		for(int ii=0; ii<yearValues.length; ii++)
			if(yearValues[ii] == curYearString) yearBox.setSelectedIndex(ii);
		
		Container schedCont = addSchedDialog.getContentPane();
		schedCont.setLayout(new BorderLayout());
		JPanel schPanel1 = new JPanel();
		JPanel schPanel2 = new JPanel();
		JPanel schPanel3 = new JPanel();
		
		schPanel1.setBackground(Color.yellow);
		schPanel2.setBackground(Color.yellow);
		schPanel3.setBackground(Color.yellow);
		
		schPanel1.add(monthBox);
		schPanel1.add(yearBox);
		
		JLabel dateLabel = new JLabel("    Date ");
		dateValueField = new WholeNumberField(curDate, 2);
		dateLabel.setLabelFor(dateValueField);
		schPanel1.add(dateLabel);
		schPanel1.add(dateValueField);
		
		JLabel timeLabel = new JLabel("     Time ");
		timeValueField = new WholeNumberField(1200,4);
		timeLabel.setLabelFor(timeValueField);
		schPanel1.add(timeLabel);
		schPanel1.add(timeValueField);
		
		JButton okButton = new JButton("OK");
		okButton.addActionListener(this);
		
		JButton cancelButton = new JButton("CANCEL");
		cancelButton.addActionListener(this);
		
		schPanel2.add(okButton);
		schPanel2.add(new JLabel("                 "));
		schPanel2.add(cancelButton);
		
		addSchText = new JTextField(40);
		addSchText.setBorder(BorderFactory.createLineBorder(Color.black));
		schPanel3.add(new JLabel("Enter Description "));
		schPanel3.add(addSchText);
		
		schedCont.add(schPanel1, BorderLayout.NORTH);
		schedCont.add(schPanel3, BorderLayout.CENTER);
		schedCont.add(schPanel2, BorderLayout.SOUTH);
	
	} // End function ConstructAddSchedDialog
	
	boolean isDateValid(int y, int m, int d)
	{
		// This function checks for incosistent date input by the user
		// in the addSchedule dialog box
		int noDays[] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
		if(IsLeapYear(y)) noDays[1]=29;
		if((d > noDays[m]))return false;
		return true;
	} // End function isDateValid
	
	boolean isTimeValid(int time)
	{
		// This function checks inconsistent time input by the user
		// based on a 24 hour clock format in the addSchedule dialog box
		if(time > 2359) return false; //Check for the upper bound
		if( (time%100) > 59) return false; // Check for the last two digits
		if( (time>0) && (time <100)) return false; // discard values from 1-99
		
		return true;
	} // End function isTimeValid()	
	
	// Implementation of MouseListener interface functions. 
	// Use of Swing bevel borders to achieve the lowered and raised date buttons
	// as the mouse is entered and exited
	public void mouseClicked(MouseEvent evt) {}
	public void mouseEntered(MouseEvent evt) {
		JButton foucsButton = (JButton) evt.getComponent();
		foucsButton.setBackground(new Color(0, 0, 128));
		foucsButton.setBorder(BorderFactory.createLoweredBevelBorder());
	}
	public void mouseExited(MouseEvent evt) {
		JButton foucsButton = (JButton) evt.getComponent();
		foucsButton.setBackground(new Color(106,90,205));	
		foucsButton.setBorder(BorderFactory.createRaisedBevelBorder());
	
	}
	public void mousePressed(MouseEvent evt) {}
	public void mouseReleased(MouseEvent evt) {}
	
} // End Class DayPlanner

class InfoDialog {
	
	// Constructor for the class InfoDialog
	public InfoDialog (JDialog parentDialog, String message)
	{
	    Object warningMessage = (Object) message;
		int OptionType = JOptionPane.DEFAULT_OPTION;
		int messageType = JOptionPane.WARNING_MESSAGE;
		
		// Throw a dialog with the above parameters over the parent window
		JOptionPane.showConfirmDialog(parentDialog, 
									  warningMessage, 
									  (String)"", 
									  OptionType, 
									  messageType);
	} // End Constructor

} // End class InfoDialog
	
