/*
 * ======================================================================
 * LibraryBuiltinFunction.java: Create library of builtin functions.
 *
 * Written by: Mark Austin                                     June, 2013
 * ======================================================================
 */

package whistle;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import whistle.builtins.*;

public class LibraryBuiltinFunction {

   // Create singleton for string-to-constant mapping ...

   public static HashMap<String,Function> s2f;

   // Constructor methods ...

   public LibraryBuiltinFunction() {
      this.s2f = new HashMap<String,Function>();
      buildLibrary();
   }

   // Retrieve library hashmap ...

   public static HashMap getStringToLibraryFunction() {
      return s2f;
   }

   // ===========================================
   // Assemble library of builtin functions ....
   // ===========================================

   public void buildLibrary() {
      String            s01;
      FunctionQuantity  fq01;
      FunctionMatrix    fm01;

      // Quantity builtin: abs function ....

      s01  = "Abs";
      fq01 = new BuiltinAbs();
      s2f.put ( s01, fq01 );

      // Quantity builtin: sqrt function ....

      s01  = "Sqrt";
      fq01 = new BuiltinSqrt();
      s2f.put ( s01, fq01 );

      // Quantity builtin: cos function ....

      s01  = "Cos";
      fq01 = new BuiltinCos();
      s2f.put ( s01, fq01 );

      // Quantity builtin: sin function ....

      s01  = "Sin";
      fq01 = new BuiltinSin();
      s2f.put ( s01, fq01 );

      // Quantity builtin: random function ....

      s01  = "Random";
      fq01 = new BuiltinRandom();
      s2f.put ( s01, fq01 );

      // Quantity builtin: max function ....

      s01  = "Max";
      fq01 = new BuiltinMax();
      s2f.put ( s01, fq01 );

      // Quantity builtin: min function ....

      s01  = "Min";
      fq01 = new BuiltinMin();
      s2f.put ( s01, fq01 );

      // Matrix builtin: matrix function ....

      s01  = "Matrix";
      fm01 = new BuiltinMatrix();
      s2f.put ( s01, fm01 );

      // Matrix builtin: solve matrix equations ....

      s01  = "Solve";
      fm01 = new BuiltinSolve();
      s2f.put ( s01, fm01 );

   }

   // =========================================================
   // Exercise methods in ConstantLibrary ....
   // =========================================================

   public static void main( String args [] ) {

      // Part 01: Create and print a simple dimension ....

      System.out.println("");
      System.out.println("*** Exercise methods in whistle/LibraryBuiltinFunction ...");
      System.out.println("*** ================================================== ...");

      LibraryBuiltinFunction lib = new LibraryBuiltinFunction ();

      System.out.println("");
      System.out.println("*** Print string-to-function mappings ...");
      System.out.println("");

      System.out.println( lib.getStringToLibraryFunction().toString() );
   }
}

