/*
 * ======================================================================
 * LibraryConstant.java: Assemble constants into a library format.
 *
 * Written by: Mark Austin
 * ======================================================================
 */

package whistle;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

public class LibraryConstant {

   // Create singleton for string-to-constant mapping ...

   public static HashMap<String,Quantity> s2c;

   // Constructor methods ...

   public LibraryConstant() {
      this.s2c = new HashMap<String,Quantity>();
      buildLibrary();
   }

   // Retrieve library hashmap ...

   public static HashMap getStringToLibraryConstant() {
      return s2c;
   }

   // ===============================================================
   // Build library of well-known unit names and their properties ...
   // ===============================================================

   public void buildLibrary() {
      Quantity q01;
      String   s01;

      // ---------------------------------------
      // SI Length units ...
      // ---------------------------------------

      s01 = "PI";
      q01 = new Quantity ( s01 );
      q01.setValue ( 3.14159265358979323846 );
      q01.setUnitsName ( "constant" );
      s2c.put ( s01, q01 );

      s01 = "E";
      q01 = new Quantity ( s01 );
      q01.setValue ( 2.71828182845904523536 );
      q01.setUnitsName ( "constant" );
      s2c.put ( s01, q01 );

      s01 = "GAMMA";
      q01 = new Quantity ( s01 );
      q01.setValue ( 0.57721566490153286060 );
      q01.setUnitsName ( "constant" );
      s2c.put ( s01, q01 );

      s01 = "DEG";
      q01 = new Quantity ( s01 );
      q01.setValue ( 57.29577951308232087680 );
      q01.setUnitsName ( "constant" );
      s2c.put ( s01, q01 );
   }

   // =========================================================
   // Exercise methods in LibraryConstant ....
   // =========================================================

   public static void main( String args [] ) {

      // Part 01: Create and print a simple dimension ....

      System.out.println("");
      System.out.println("*** Exercise methods in whistle/LibraryConstant ...");
      System.out.println("*** ============================================= ...");

      LibraryConstant lib = new LibraryConstant ();

      System.out.println("");
      System.out.println("*** Print string-to-constants mapping ...");
      System.out.println("");

      System.out.println( lib.getStringToLibraryConstant().toString() );
   }
}

